import { getGistApi } from './gistApi';
import Cryptr from 'cryptr';
export declare enum CompressionType {
    msgpack = "msgpack",
    none = "none",
    pretty = "pretty"
}
export interface GistDatabaseOptions {
    compression?: CompressionType;
    description?: string;
    encryptionKey?: string;
    id?: string;
    public?: boolean;
    token: string;
}
export type GistResponse = {
    files: Record<string, {
        content: string;
        size: number;
    }>;
    id: string;
    url: string;
};
export interface ExtraFile {
    content?: string;
    gist?: GistResponse;
    id: string;
    url: string;
}
export type ExtraFiles = Record<string, ExtraFile>;
export type Doc<T = any> = {
    extraFile: ExtraFile;
    files: ExtraFiles;
    gist: {
        [key: string]: any;
        id: string;
    };
    id: string;
    rev: string;
    value: T;
};
export type DocRef = {
    id: string;
    ttl: {
        createdAt: number;
        ttl: number;
    };
};
export declare const defaultOptions: Partial<GistDatabaseOptions>;
export declare class GistDatabase {
    private readonly options;
    readonly gistApi: ReturnType<typeof getGistApi>;
    static MAX_FILE_SIZE_BYTES: number;
    static MAX_FILES_PER_GIST: number;
    isNewDatabase: boolean;
    initialized: boolean;
    private readonly cryptr;
    static ROOT_GIST_NAME: string;
    constructor(options: GistDatabaseOptions);
    getDatabaseId(): string;
    static createDatabaseRoot(options: GistDatabaseOptions): Promise<GistResponse>;
    init(): Promise<any>;
    private initIfNeeded;
    keys(): Promise<string[]>;
    getRoot(): Promise<GistResponse>;
    get<T = any>(key: string | string[], { rev }?: {
        rev?: string;
    }): Promise<Doc<T>>;
    getMany(keys: string[]): Promise<Doc[]>;
    has(key: string | string[]): Promise<boolean>;
    static unpack(files: GistResponse['files'], type: CompressionType, cryptr: Cryptr): {};
    static pack(path: any, value: any, { ttl, createdAt, rev, extraFile }: {
        createdAt?: number;
        extraFile?: ExtraFile;
        rev?: string;
        ttl?: number;
    }, type: CompressionType, cryptr: Cryptr): Promise<Record<string, {
        content: string;
    }>>;
    set<T = any>(key: string | string[], args: {
        description?: string;
        files?: Record<string, {
            content: string;
        }>;
        rev?: string;
        ttl?: number;
        value?: T;
    }): Promise<Doc<T>>;
    delete(key: string | string[]): Promise<any>;
    deleteMany(keys: string[]): Promise<any[]>;
    destroy(): Promise<void>;
    private deleteExtraFilesForGist;
    static get<T = any>(obj: T, path: string[]): T;
    static set<T = any>(obj: T, path: string[], value: any): T;
    static del<T>(obj: T, path: string[]): T;
    static ttlIsExpired(ttl: DocRef['ttl']): boolean;
    static formatPath(path: string[], index?: number): string;
    static serialize(value: any, type: CompressionType, cryptr: Cryptr): string;
    static deserialize(value: any, type: CompressionType, cryptr: Cryptr): any;
    static rev(): string;
    static formatRevisionError(expected: string, received: string): string;
}
